/*
 * dh6_adm.h    DHCPv6 Administration Command Formats
 *
 * Version:     @(#)dh6_adm.h   1.0     Dec-8-96
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed WITHOUT ANY WARRANTY and even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Many thanks to Jim Bound in Digital Equipment Corp.
 * Appreciate any further work on this release.
 *
 * Authors:     Yunzhou Li (yunzhou@iol.unh.edu)
 *
 */

#ifndef __DH6_ADM_H
#define __DH6_ADM_H

/*
 * The UNIX server names for each interface of a DHCPv6 client are:
 *
 *	dh6cli-icmp.<interface name>
 *	dh6cli-adm.<interface name>
 *
 * Below is the server path. Every socket will be referenced by
 * the server path and a server name.
 */
#define DH6_SERV_PATH  	"/usr/local/etc/dhcpv6/"  

#define DH6_TMP        	"/tmp/dhcpv6.XXXXXX"

/*
 *  message
 */

typedef union dh6_message {
    struct {
        u_int8_t    type;
    }                       msg;
    struct dh6_solicit      sol;
    struct dh6_advertise    adv;
    struct dh6_request      req;
    struct dh6_reply        rpl;
    struct dh6_release      rel;
    struct dh6_reconfigure  conf;
} DH6_MESSAGE;

#define MAX_MSG_LEN     256	/* ???? */

/*
 *	administration command format
 */

typedef struct {
    u_int8_t		cmd;
    u_int8_t		bit7,
			bit6,
			bit5,
			bit4,
			bit3,
			bit2,
			bit1,
			bit0;
    u_int16_t		len;
    union {
    	u_int8_t		data[0];
	struct {
		u_int16_t	lport;
		u_int16_t	fport;
		struct in6_addr	laddr;
		struct in6_addr	faddr;
		DH6_MESSAGE	msg;
	}			dh6;
#define ADM_DH6MSG_HDRLEN	(4 + 2*sizeof(struct in6_addr))
	struct {
		struct in6_addr lladdr;
		struct in6_addr agent;
		struct in6_addr client;
	}			sdb;
#define ADM_SVRDB_LEN		(3*sizeof(struct in6_addr))
	struct {
		struct in6_addr server;
		struct in6_addr agent;
		struct in6_addr client;
	}			cdb;
#define ADM_CLIDB_LEN		(3*sizeof(struct in6_addr))
    } un;
} ADM_MESSAGE;

/*
 *  Administration command value
 */
#define ADM_CMD_START       	0
#define ADM_CMD_RESTART     	1
#define ADM_CMD_STOP        	2

#define ADM_OPEN_DHCPV6 	8
#define ADM_CLOSE_DHCPV6	9

#define ADM_DH6_MSG		16
#define ADM_SHOW_DB		17

extern int debug;

#endif /* __DH6_ADM_H */
