/*
 * dh6_cli.h    DHCPv6 Client Data Structures
 *
 * Version:     @(#)dh6_cli.h   1.0     Dec-8-96
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed WITHOUT ANY WARRANTY and even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Many thanks to Jim Bound in Digital Equipment Corp.
 * Appreciate any further work on this release.
 *
 * Authors:     Yunzhou Li (yunzhou@iol.unh.edu)
 *
 */

#ifndef __DH6_CLI_H
#define __DH6_CLI_H

#define DH6CLI_CONF	"dh6_client.conf"
#define DH6CLI_BOOT	"dh6_client.boot"
#define CLI_CONF_PATH	"/etc/"

/*
 *	Relay list
 */
struct cli_relay {
    struct cli_relay	*next;
    struct cli_relay	*prev;

    struct in6_addr 	 cr_addr;  	/* Link-local address (net-order)*/  
    struct cli_server	*cr_issvr;	/* pointer to a server if it is */
};

/*
 *	Server list
 */
struct cli_server {
    struct cli_server	*next;
    struct cli_server	*prev;

    struct in6_addr 	 cs_addr;    	/* address (net-order) */
    struct cli_relay	*cs_rly;	/* relay that it is associated with */
};

/* 
 *	Client address list
 */
struct cli_ifaddr {
    struct cli_ifaddr   *next;
    struct cli_ifaddr   *prev;

    IF_INFO		*cia_ifi;	/* interface information */
    struct cli_server	*cia_svr;	/* server serving for the client */
    struct cli_relay 	*cia_rly;	/* agent on the same link */
    struct in6_addr      cia_addr;      /* address (net order) */
    u_int8_t             cia_prfsize;   /* prefix (net order) */

#define CIA_MSG_NONE  	 0x00		/* No messages */
#define CIA_REQ_WAIT	 0x01		/* Request sent and waiting */
#define CIA_RPL_MORE	 0x02		/* waiting more replies */
#define CIA_REQ_CLRD	 0x04		/* Request cleared */
#define CIA_REL_WAIT	 0x06		/* Release sent and waiting */
#define CIA_REL_CLRD	 0x08		/* Release cleared */

#define CIA_ADDR_NONE	 0x00		/* No address associated */
#define CIA_ADDR_TMP 	 0x10		/* Tentative address */
#define CIA_ADDR_PRFD	 0x20		/* preferred address */
#define CIA_ADDR_VALID   0x40		/* valid address */
#define CIA_ADDR_INVAL   0x60		/* invalid address */
    u_int8_t             cia_state; 	/* Client state */

    u_int16_t       	 cia_id;     	/* transaction ID (host-order) */
    u_int32_t		 cia_req_life;	/* requested lifetime (host-order) */
    u_int32_t            cia_valid_life;/* seconds (host order) */
    u_int32_t            cia_prefer_life;/* seconds (host order) */

    struct fd_list	 cia_sk;	/* UDP socket for this client */
    struct u_timer       cia_xmit_timer;/* transmission timer */
    u_int8_t             cia_xmit_rc; 	/* retransmission counter */
					/* transmission time can be returned
					   by stop_timer() */

    struct u_timer       cia_life_timer;/* lifetime Timer */
    int			 cia_extlen;	/* length of extension */
    void		*cia_ext;	/* extensions */
};

struct cli_record {
    char   		 ifname[IFNAMSIZ];
    struct in6_addr	 svr;		/* server serving for the client */
    struct in6_addr 	 rly;		/* agent on the same link */
    struct in6_addr      addr;      	/* address (net order) */
    u_int8_t             prfsize;   	/* prefix (net order) */
    u_int8_t             state; 	/* Client state */
    u_int16_t       	 id;     	/* transaction ID (host-order) */
    u_int32_t		 req_life;	/* requested lifetime (host-order) */
    u_int32_t            valid_life;	/* seconds (host order) */
    u_int32_t            prefer_life;	/* seconds (host order) */
};

extern inline int
cia_addr_verified(struct cli_ifaddr *cia)
{
	return( (cia->cia_state & CIA_ADDR_PRFD) || 
		(cia->cia_state & CIA_ADDR_VALID) );
}

extern struct cli_relay * cr_create( void );
extern void cr_destroy( struct cli_relay *);
extern struct cli_relay * cr_get( struct in6_addr *);
extern struct cli_server * cs_create( void );
extern void cs_destroy( struct cli_server *);
extern struct cli_server * cs_get( struct in6_addr *);
extern struct cli_ifaddr * cia_create( void );
extern void cia_destroy( struct cli_ifaddr *);
extern struct cli_ifaddr * cia_get( struct in6_addr *);
extern struct cli_ifaddr * 
cia_get_by_pairaddrs( struct in6_addr *, struct in6_addr *);

#endif /* __DH6_CLI_H */
