/*
 * dh6_clicmd.c DHCPv6 Client Command
 *
 * Version:     @(#)dh6_clicmd.c   1.0     Dec-8-96
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed WITHOUT ANY WARRANTY and even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Many thanks to Jim Bound in Digital Equipment Corp.
 * Appreciate any further work on this release.
 *
 * Authors:     Yunzhou Li (yunzhou@iol.unh.edu)
 *
 */

#include "compat.h"
#include "localdefs.h"
#include "u_timer.h"
#include "dispatch.h"
#include "u_buf.h"
#include "dh6.h"
#include "ext6.h"
#include "dh6_if.h"
#include "dh6_cli.h"
#include "dh6_adm.h"
#include "sock.h"

extern char *optarg;
extern int optind, opterr, optopt;

int debug = 0;

/*
 *  Generate an error message with the printf facility.
 */
void p_error (int err, const char *ctl, ...)
{
    	char buf[1000];
    	va_list list;
    	va_start (list,ctl);
    	vsprintf (buf,ctl,list);
    	printf("error %d: %s\n", err, buf);
}


main( int argc, char * argv[] )
{
        int     optval;
        char    op;
	u_int8_t buf[MAX_MSG_LEN];
	ADM_MESSAGE *msg=(ADM_MESSAGE *)buf;
	DH6_MESSAGE *dh6=(DH6_MESSAGE *)(&msg->un.dh6.msg);
	char ifname[IFNAMSIZ];
	int  if_flag = 0;
	struct sockaddr_un  su_svr, if_svr;
	int  sockfd, su_svrlen, if_svrlen, msglen;
	IF_INFO *ifi;

        opterr = 0; 
        while ((op = getopt(argc-1, argv+1, "d:D:i:I:")) != EOF) {
                switch (op) {
#ifdef DH6_DEBUG
                case 'd':   /* debug */
                case 'D':   /* debug */
                	debug = atoi(optarg);
                        break;
#endif
		case 'i':
		case 'I':   /* interface */
			strcpy(ifname, optarg);
			if_flag = 1;
			break;
                default:
                }
        }

	if ( argc<2 )
		cmd_usage();

	if ( strcmp(argv[1],"start")==0 ) {
		char cmd[50];
		sprintf(cmd, "%sdh6cd -d%d", DH6_SERV_PATH, debug);
		system(cmd);
		exit(0);
	}
	else if ( strcmp(argv[1],"restart")==0 ) {
		char cmd[50];
		sprintf(cmd, "%s stop", argv[0]);
		system(cmd);
		sleep(1);
		sprintf(cmd, "%s start -d%d", argv[0], debug);
		system(cmd);
		exit(0);
	}

	else if ( strcmp(argv[1],"stop")!=0 &&
	          strcmp(argv[1],"open")!=0 &&
	          strcmp(argv[1],"close")!=0 &&
	          strcmp(argv[1],"dbshow")!=0 &&
	          strcmp(argv[1],"solicit")!=0 &&
	          strcmp(argv[1],"request")!=0 &&
	          strcmp(argv[1],"release")!=0 )
		cmd_usage();

	bzero((char *) &su_svr, sizeof(su_svr));
	su_svr.sun_family = AF_UNIX;
	strcpy(su_svr.sun_path, dh6cli_adm_name());
	su_svrlen = strlen(su_svr.sun_path) + sizeof(su_svr.sun_family) + 1;

	bzero((char *) &if_svr, sizeof(if_svr));
	if_svr.sun_family = AF_UNIX;
	strcpy(if_svr.sun_path, dh6cli_adm_server_name(ifname));
	if_svrlen = strlen(if_svr.sun_path) + sizeof(if_svr.sun_family) + 1;

	/*
	 * 	Open a socket (an UNIX domain stream socket).
	 *	and connect to server later
	 */
	if ( (sockfd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
		p_error(1201, strerror(errno));

	memset(buf, 0, MAX_MSG_LEN);
    	if ( strcmp(argv[1],"stop")==0 ) {
		if (connect(sockfd, (struct sockaddr *)&su_svr, su_svrlen) < 0)
			p_error(1202, strerror(errno));

   		msg->cmd = ADM_CMD_STOP;
   		msg->len = 0;
		msglen = 4;
		cmd_sendto_serv( sockfd, msg, msglen );
		goto out;
    	}

	if ( strcmp(argv[1],"open")==0 ) {
		if (connect(sockfd, (struct sockaddr *)&su_svr, su_svrlen) < 0)
			p_error(1203, strerror(errno));

   		msg->cmd = ADM_OPEN_DHCPV6;
   		msg->len = strlen(argv[2]);
   		strcpy(msg->un.data, argv[2]);

		msglen = 4 + msg->len;
		cmd_sendto_serv( sockfd, msg, msglen );
		goto out;
	}

	if ( !if_flag ) {
		p_error(1204, "interface name missing");
		goto out;
	}

        u_free_list_init();  
        init_ifi_table();
        get_ifi_table();

	if ((ifi=seek_ifi_byname(ifname, strlen(ifname))) == NULL) {
		p_error(1205, "interface not found");
		goto out;
	}

	if ( strcmp(argv[1],"close")==0 ) {
		if (connect(sockfd, (struct sockaddr *)&if_svr, if_svrlen) < 0)
			p_error(1206, strerror(errno));

                msg->cmd = ADM_CMD_STOP;
                msg->len = 0;
                msglen = 4;
                cmd_sendto_serv( sockfd, msg, msglen );
                goto out;
	}

        opterr = 0;
    	if ( strcmp(argv[1],"dbshow")==0 ) {
                struct in6_addr addr;

		if (connect(sockfd, (struct sockaddr *)&if_svr, if_svrlen) < 0)
			p_error(1206, strerror(errno));

                msg->cmd = ADM_SHOW_DB;
                while ((op=getopt(argc-1,argv+1,"s:S:a:A:v:V:"))!=EOF) {
                        switch (op) {
                        case 's':
                        case 'S':
                                inet_pton(AF_INET6, optarg, &addr);
                		memcpy(&msg->un.cdb.server, &addr, 
					IPV6_ADDR_LEN);
				msg->bit0 = 1;
				break;
                        case 'a':
                        case 'A':
                                inet_pton(AF_INET6, optarg, &addr);
                		memcpy(&msg->un.cdb.agent, &addr, 
					IPV6_ADDR_LEN);
				msg->bit1 = 1;
				break;
                        case 'v':
                        case 'V':
                                inet_pton(AF_INET6, optarg, &addr);
                		memcpy(&msg->un.cdb.client, &addr, 
					IPV6_ADDR_LEN);
				msg->bit2 = 1;
				break;
			}
		}
                msg->len = ADM_CLIDB_LEN;
                msglen = 4 + msg->len;
                cmd_sendto_serv( sockfd, msg, msglen );
                goto out;
    	}
    	if ( strcmp(argv[1],"solicit")==0 ) {
		int Pbit = 0, Cbit = 0;
		if (connect(sockfd, (struct sockaddr *)&if_svr, if_svrlen) < 0)
			p_error(1207, strerror(errno));

        	while ((op = getopt(argc-1, argv+1, "pc")) != EOF) {
            		switch (op) {
			case 'p':
			case 'P':
                                Pbit = 1; break;
			case 'c':
			case 'C':
                                Cbit = 1; break;
			}
		}

		msg->cmd = ADM_DH6_MSG;
		msg->un.dh6.lport = DH6_CLIENT_PORT;
		msg->un.dh6.fport = DH6_AGENT_PORT;
		memcpy(&msg->un.dh6.laddr, ifi_lladdr(ifi), IPV6_ADDR_LEN);
		memcpy(&msg->un.dh6.faddr, dh6sin_agent_group(), IPV6_ADDR_LEN);

	        dh6->sol.type = DH6_SOLICIT;
        	dh6->sol.Pbit = Pbit; 
        	dh6->sol.Cbit = Cbit;
		msg->len = sizeof(struct dh6_solicit) + ADM_DH6MSG_HDRLEN;
		msglen = msg->len + 4;
		cmd_sendto_serv( sockfd, msg, msglen );
		goto out;
    	}
    	if ( strcmp(argv[1],"request")==0 ) {
		struct in6_addr server, agent, client;
		u_int8_t *ext = NULL;
		int extlen = 0;
		int prefer = 0, valid = 0, Qbit = 0, v_flag = 0;

		if (connect(sockfd, (struct sockaddr *)&if_svr, if_svrlen) < 0)
			p_error(1208, strerror(errno));

		msg->cmd = ADM_DH6_MSG;
		msg->un.dh6.lport = DH6_CLIENT_PORT;
		msg->un.dh6.fport = DH6_AGENT_PORT;
		memcpy(&msg->un.dh6.laddr, ifi_lladdr(ifi), IPV6_ADDR_LEN);

	        dh6->req.type = DH6_REQUEST;    
        	while ((op=getopt(argc-1,argv+1,"s:S:a:A:v:V:l:L:cCqQ"))!=EOF) {
            		switch (op) {
			case 's':
			case 'S':
        			dh6->req.Sbit = 1;
                                inet_pton(AF_INET6, optarg, &server);
				set_request_server_addr(&dh6->req, &server);
				break;
			case 'a':
			case 'A':
                                inet_pton(AF_INET6, optarg, &agent);
				set_request_agent_addr(&dh6->req, &agent);
				memcpy(&msg->un.dh6.faddr, &agent, 
					IPV6_ADDR_LEN);
				break;
			case 'v':
			case 'V':
                                inet_pton(AF_INET6, optarg, &client);
				v_flag = 1;
				break;
			case 'l':
        			prefer = atoi(optarg); 
				break;
			case 'L':
        			valid = atoi(optarg); 
				break;
			case 'c':
			case 'C':
        			dh6->req.Cbit = 1;
				break;
			case 'q':
			case 'Q':
        			Qbit = 1;
				break;
			}
		}
		set_request_llocal_addr(&dh6->req, ifi_lladdr(ifi));

		msg->len = ADM_DH6MSG_HDRLEN + sizeof(struct dh6_request) +
              		 IPV6_ADDR_LEN*(dh6->req.Sbit+2); 

		append_addr_ext6(ext, &extlen, (v_flag) ? &client : NULL,
			0, prefer, valid, NULL, Qbit, 0, 0); 
	        if ( extlen != 0 && ext != NULL ) {
        	        memcpy(buf+4+msg->len, ext, extlen);
        	        msg->len += extlen;
        	}

		msglen = msg->len + 4;
		cmd_sendto_serv( sockfd, msg, msglen );
		goto out;
    	}
    	if ( strcmp(argv[1],"release")==0 ) {
		struct in6_addr server, agent;
		if (connect(sockfd, (struct sockaddr *)&if_svr, if_svrlen) < 0)
			p_error(1209, strerror(errno));

		msg->cmd = ADM_DH6_MSG;
		msg->un.dh6.lport = DH6_CLIENT_PORT;
		msg->un.dh6.fport = DH6_AGENT_PORT;
		memcpy(&msg->un.dh6.laddr, ifi_lladdr(ifi), IPV6_ADDR_LEN);

	        dh6->rel.type = DH6_RELEASE;    
        	while ((op=getopt(argc-1,argv+1,"s:S:a:A:tT")) != EOF) {
            		switch (op) {
			case 's':
			case 'S':
                                inet_pton(AF_INET6, optarg, &server);
				memcpy(&msg->un.dh6.faddr, &server, 
						IPV6_ADDR_LEN);
				break;
			case 'a':
			case 'A':
                                inet_pton(AF_INET6, optarg, &agent);
        			memcpy(&dh6->rel.AgtAddr, &agent, 
						IPV6_ADDR_LEN);
				break;
			case 'x':
			case 'X':
				dh6->rel.id = htons(atoi(optarg));
				break;
			case 't':
			case 'T':
        			dh6->rel.Dbit = 1;
				break;
			}
		}
        	memcpy(&dh6->rel.lladdr, ifi_lladdr(ifi), IPV6_ADDR_LEN);

		msg->len = ADM_DH6MSG_HDRLEN + dh6_release_msg_len(&dh6->rel);
		msglen = msg->len + 4;
		cmd_sendto_serv( sockfd, msg, msglen );
		goto out;
    	}

out:
	close(sockfd);
	exit(0);
}

cmd_usage( void )
{
	printf("usage: dh6cc start -d<deubg>\n");
	printf("       dh6cc restart -d<debug>\n");
	printf("       dh6cc open -i<ifname>\n");
	printf("       dh6cc close -i<ifname>\n");
        printf("       dh6sc dbshow -s<server> -a<agent> -v<client>\n");
	printf("       dh6cc solicit -i<ifname> -p -c\n");
	printf("       dh6cc request -i<ifname> -s<server> -a<agent> -c -v<client> -q -l<prefer> -L<valid>\n");
	printf("       dh6cc release -i<ifname> -s<server> -a<agent> -x<id> -t\n");
	printf("       dh6cc stop\n");
	exit(1);
}

/*
 *	send a message to server
 */

int cmd_sendto_serv( int sockfd, ADM_MESSAGE *msg, int msglen )
{
	if ( send(sockfd, msg, msglen, 0) != msglen)
	{
		p_error(1210, strerror(errno));
		return -1;
	}

	return 0;
}
