/*
 * dh6_rlycmd.c DHCPv6 Relay Command
 *
 * Version:     @(#)dh6_rlycmd.c   1.0     Dec-8-96
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed WITHOUT ANY WARRANTY and even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Many thanks to Jim Bound in Digital Equipment Corp.
 * Appreciate any further work on this release.
 *
 * Authors:     Yunzhou Li (yunzhou@iol.unh.edu)
 *
 */

#include "compat.h"
#include "localdefs.h"
#include "u_timer.h"
#include "dispatch.h"
#include "u_buf.h"
#include "dh6.h"
#include "ext6.h"
#include "dh6_if.h"
#include "dh6_rly.h"
#include "dh6_adm.h"
#include "sock.h"

extern char *optarg;
extern int optind, opterr, optopt;

int debug = 0;

/*
 *  Generate an error message with the printf facility.
 */
void p_error (int err, const char *ctl, ...)
{
    	char buf[1000];
    	va_list list;
    	va_start (list,ctl);
    	vsprintf (buf,ctl,list);
    	printf("error %d: %s\n", err, buf);
}


main( int argc, char * argv[] )
{
        int     optval;
        char    op;
        u_int8_t buf[MAX_MSG_LEN];
        ADM_MESSAGE *msg=(ADM_MESSAGE *)buf;
        DH6_MESSAGE *dh6=(DH6_MESSAGE *)(&msg->un.dh6.msg);
        struct sockaddr_un  serv_addr;
        int  sockfd, servlen, msglen;

	if ( argc<2 )
		cmd_usage();

        while ((op = getopt(argc-1, argv+1, "d:D:")) != EOF) {
                switch (op) {
#ifdef DH6_DEBUG
                case 'd':   /* debug */
                case 'D':   /* debug */
                        debug = atoi(optarg);
                        break;
#endif
                default:
                }
        }

        if ( strcmp(argv[1],"start")==0 ) {
                char cmd[50];
                sprintf(cmd, "%sdh6rd -d%d", DH6_SERV_PATH, debug);
                system(cmd);
                exit(0);
        }
        else if ( strcmp(argv[1],"restart")==0 ) {
                char cmd[50];
                sprintf(cmd, "%s stop", argv[0]);
                system(cmd);
                sleep(1);
                sprintf(cmd, "%s start -d%d", argv[0], debug);
                system(cmd);
                exit(0);
        }

	else if ( strcmp(argv[1],"stop")!=0 &&
	          strcmp(argv[1],"solicit")!=0 )
		cmd_usage();

	bzero((char *) &serv_addr, sizeof(serv_addr));
	serv_addr.sun_family = AF_UNIX;
	strcpy(serv_addr.sun_path, dh6rly_adm_server_name());
	servlen = strlen(serv_addr.sun_path) + sizeof(serv_addr.sun_family) + 1;

	if ( (sockfd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
		p_error(2201, strerror(errno));

	if (connect(sockfd, (struct sockaddr *) &serv_addr, servlen) < 0)
		p_error(2202, strerror(errno));

	memset(buf, 0, MAX_MSG_LEN);
        if ( strcmp(argv[1],"stop")==0 ) {
                msg->cmd = ADM_CMD_STOP;
                msg->len = 0;
                msglen = 4;
                cmd_sendto_serv( sockfd, msg, msglen );
                goto out;
        }
        if ( strcmp(argv[1],"solicit")==0 ) {
                struct in6_addr addr;

                msg->cmd = ADM_DH6_MSG;
                msg->un.dh6.lport = DH6_AGENT_PORT;
                msg->un.dh6.fport = DH6_AGENT_PORT;

        	dh6->sol.type = DH6_SOLICIT;
        	dh6->sol.Cbit = 0;
        	dh6->sol.Lbit = 0;
        	dh6->sol.Abit = 1;
        	dh6->sol.Pbit = 0;

                while ((op=getopt(argc-1,argv+1,"s:S:a:A:"))!=EOF) {
                        switch (op) {
                        case 's':
                        case 'S':
                                inet_pton(AF_INET6, optarg, &addr);
                                memcpy(&msg->un.dh6.faddr, &addr,
                                        IPV6_ADDR_LEN);
                                break;
                        case 'a':
                        case 'A':
                                inet_pton(AF_INET6, optarg, &addr);
                                memcpy(&msg->un.dh6.laddr, &addr,
                                        IPV6_ADDR_LEN);
			   	set_solicit_relay_addr(&dh6->sol, &addr);
                                break;
                        }
                }

                msg->len = ADM_DH6MSG_HDRLEN + dh6_solicit_msg_len(&dh6->sol);
                msglen = msg->len + 4;
                cmd_sendto_serv( sockfd, msg, msglen );
                goto out;
        }


out:
	close(sockfd);
	exit(0);
}

cmd_usage( void )
{
	printf("usage: dh6rc start -d<deubg>\n");
	printf("       dh6rc restart -d<debug>\n");
	printf("       dh6rc stop\n");
	printf("       dh6rc solicit -s<server> -a<agent>\n");
	exit(1);
}

/*
 *	send a message to server
 */

int cmd_sendto_serv( int sockfd, ADM_MESSAGE *msg, int msglen )
{
	if ( send(sockfd, msg, msglen, 0) != msglen)
	{
		p_error(2203, strerror(errno));
		return -1;
	}

	return 0;
}
