/*
 * dh6_svr.h    DHCPv6 Server Data Structures
 *
 * Version:     @(#)dh6_svr.h   1.0     Dec-8-96
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed WITHOUT ANY WARRANTY and even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Many thanks to Jim Bound in Digital Equipment Corp.
 * Appreciate any further work on this release.
 *
 * Authors:     Yunzhou Li (yunzhou@iol.unh.edu)
 *
 */

#ifndef __DH6_SVR_H
#define __DH6_SVR_H

#define SVR_DB_CHECK_INTERVAL	60	/* db is checked every 60 seconds */

#define DH6_SVR_INUSE	"dh6_svr_inuse.db"
#define DH6_SVR_INVAL	"dh6_svr_inval.db"
#define DH6_NEWADDR	"dh6_newaddress"
#define DH6SVR_CONF	"dh6_server.conf"
#define SVR_CONF_PATH	"/etc/"

/*
 *	Client list
 */
struct svr_client {
    struct svr_client	*next;
    struct svr_client	*prev;

    struct svr_ifaddr	*sc_sia;	/* a server address */
    struct in6_addr 	 sc_lladdr;     /* client link local (net-order) */
    struct in6_addr 	 sc_rlyaddr;    /* relay address (net-order) */
    struct in6_addr 	 sc_cliaddr;    /* client address (net-order) */
    u_int16_t       	 sc_id;     	/* transaction ID (host-order) */
    u_int8_t		 sc_prfsize; 	/* prefix size */
#define SC_STATE_NONE	 0		/* no address */
#define SC_STATE_TEMP	 1		/* temporary address */
#define SC_STATE_PREFER  2		/* preferred address */
#define SC_STATE_VALID   3		/* valid address */
#define SC_STATE_DELETE  4		/* invalid address */
    u_int8_t             sc_state; 	/* Client state */
    u_int32_t            sc_valid_life;	/* seconds (host order) */
    u_int32_t            sc_prefer_life;/* seconds (host order) */
    struct u_timer	 sc_xmit_timer; /* transmission timer */
    u_int16_t            sc_xmit_timeout;/*remaining time */
    struct u_timer	 sc_dns_timer;  /* timer for updating DNS */
};

/* 
 *	Server Address list
 */
struct svr_ifaddr {
    struct svr_ifaddr   *next;
    struct svr_ifaddr   *prev;

    IF_INFO		*sia_ifi;	/* interface information */
    struct in6_addr      sia_addr;      /* address (net order) */
    UNION_LIST		 sia_clilist;	/* client hash table */
    struct fd_list	 sia_sk;	/* socket for server address */
#define SERVER_DEFAULT_LIFETIME	1
    u_int8_t		 sia_lifetime;  /* server advertisement lifetime */
};

struct svr_record {
    struct in6_addr 	 svraddr;     	/* client link local (net-order) */
    struct in6_addr 	 lladdr;     	/* client link local (net-order) */
    struct in6_addr 	 rlyaddr;    	/* relay address (net-order) */
    struct in6_addr 	 cliaddr;    	/* client address (net-order) */
    u_int16_t       	 id;     	/* transaction ID (host-order) */
    u_int8_t		 prfsize; 	/* prefix size */
    u_int8_t             state; 	/* Client state */
    u_int32_t            prefer_life;	/* seconds (host order) */
    u_int32_t            valid_life;	/* seconds (host order) */
};

extern struct svr_client * sc_create( void );
extern inline void sc_destroy( struct svr_client *);
extern struct svr_client * sc_get( struct svr_ifaddr *, struct in6_addr	*, 
		struct in6_addr	*);
extern struct svr_ifaddr * sia_create( void );
extern inline void sia_destroy( struct svr_ifaddr *sia );
extern struct svr_ifaddr * sia_get(struct in6_addr *addr);

#endif /* __DH6_SVR_H */
