/*
 * su_cli.c    	DHCPv6 Client Super Daemon
 *
 * Version:     @(#)su_cli.c   1.0     Dec-8-96
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed WITHOUT ANY WARRANTY and even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Many thanks to Jim Bound in Digital Equipment Corp.
 * Appreciate any further work on this release.
 *
 * Authors:     Yunzhou Li (yunzhou@iol.unh.edu)
 *
 */

#include "compat.h"
#include "localdefs.h"
#include "u_timer.h"
#include "dispatch.h"
#include "u_buf.h"
#include "dh6.h"
#include "ext6.h"
#include "dh6_if.h"
#include "dh6_cli.h"
#include "dh6_adm.h"
#include "sock.h"

extern void su_run_unix_server( void * );
extern void su_icmp_msgrcv( void * );
extern void su_adm_msgrcv( void * );

int debug = 0;

struct fd_list icmp_sock, unix_sock;
struct poll_list poll_unix;
int  quit_flag = 0;
extern UNION_LIST 	ifi_table;

static inline u_int8_t rtadv_managed_bit(struct icmpv6 *icmp_hdr)
{
	return (icmp_hdr->icmp6_hun.ih6_rtad.irta_aflg & 0x80) >> 7;
}

static inline u_int8_t rtadv_other_bit(struct icmpv6 *icmp_hdr)
{
	return (icmp_hdr->icmp6_hun.ih6_rtad.irta_aflg & 0x40) >> 6;
}

main(int argc, char *argv[])
{
        int 	optval;
	char 	op;
#ifdef ICMPV6_FILTER
	struct icmp6_filter icm6f;
#endif

        opterr = 0;
        while ((op = getopt(argc, argv, "d:D:")) != EOF) {
            	switch (op) {
#ifdef DH6_DEBUG
            		case 'd':   /* debug */
            		case 'D':   /* debug */
                		debug = atoi(optarg);
                		break;
#endif
			default:
		}
	}

        if ( daemon_init() < 0 )
                log_fatal(1001, "daemon init fail");

	if (debug > 0)
    		openlog ("", LOG_CONS, LOG_DAEMON);
	else
    		openlog (argv[0], 0, LOG_DAEMON);

	u_free_list_init();
	begin_timing();

	init_ifi_table();
	get_ifi_table();

#ifdef DH6_DEBUG
        if ( look_cache() ) {
		log_info("debug = %d", debug);
		print_ifi_table();
	}
#endif

        /* open unix socket; listen for incoming administration commands. */
        if ( (unix_sock.fd=open_unix_sock(dh6cli_adm_name())) < 0 )
                log_fatal(1002, "open unix socket fail");

	init_poll(&poll_unix);
	poll_unix.data = &unix_sock;
	poll_unix.function = su_run_unix_server;
	add_poll(&poll_unix);

#ifdef 0 /* very strange "resource temporily unavailable"!!! */
	init_fdp(&unix_sock);
	unix_sock.data = &unix_sock;
	unix_sock.function = su_run_unix_server;
	add_fdp(&unix_sock);
#endif
 
        /* Open a raw ICPMv6 socket */
        if ((icmp_sock.fd = socket(PF_INET6, SOCK_RAW, IPPROTO_ICMPV6)) < 0)
                log_sys(1003, "socket");

        optval = 1;
        if (setsockopt(icmp_sock.fd, IPPROTO_IPV6, IPV6_RXINFO,
			(char *)&optval, sizeof(optval)) < 0)
                log_fatal(3006, "IPV6_RXINFO fail");

	init_fdp(&icmp_sock);

#ifdef ICMPV6_FILTER
	ICMPV6_FILTER_SETBLOACKALL(&icm6f);
	ICMPV6_FILTER_SETPASS(ICMPV6_ADVERTISMENT_RT, &icm6f);
        if (setsockopt(icmp_sock.fd, IPPROTO_ICMPV6, ICMPV6_FILTER, (char *)
                       &icm6f, sizeof(icm6f)) < 0)
                log_fatal(1005,"ICMPV6_FILTER fail");
#endif

	init_fdp(&icmp_sock);
	icmp_sock.data = &icmp_sock;
	icmp_sock.function = su_icmp_msgrcv;
	add_fdp(&icmp_sock);

        for (;;) {
		dispatch(); 
		if ( quit_flag )
			break;
        }
 
	end_timing();
	del_poll(&poll_unix);
	close_unix_sock(unix_sock.fd, dh6cli_adm_name());
	del_fdp(&icmp_sock); close(icmp_sock.fd);
#ifdef DH6_DEBUG
        if ( look_xmit() )
		log_info("exit from %s", argv[0]);
#endif
	closelog();
	exit(0);
}

void su_run_unix_server( void * data )
{
	struct fd_list 		*p = (struct fd_list *)data, *fdp;
	int			fd;
        struct sockaddr_un 	unix_addr;
        int  			unix_len = sizeof(unix_addr);

	for (;;) {
        	fd = accept(p->fd, (struct sockaddr *)&unix_addr, &unix_len);
                if (fd < 0) {
                        if ( errno!=EWOULDBLOCK )
                                log_sys(1006, "accept");
			else
				return;  /* no more pending connections */
                } else {
			if ( (fdp=malloc(sizeof(struct fd_list))) == NULL )
				log_sys(1007, "malloc");
        		init_fdp(fdp); 
			fdp->fd   = fd;
        		fdp->data = fdp;
        		fdp->function = su_adm_msgrcv;
			add_fdp(fdp);
                }
	}
}

void su_adm_msgrcv( void * data )
{
	struct fd_list *p = (struct fd_list *)data;
	struct if_info *ifi;
        ADM_MESSAGE  msg;
        int ret;
 
        for (;;) {
                ret = recv(p->fd, (char*)&msg, MAX_MSG_LEN, 0);
                if ( ret==0 )
                        return;
                else if (ret < 0) {
                	if (errno == EINTR)
                         	continue;
			else
                        	log_sys(1008, "recv");
		}
         
#ifdef DH6_DEBUG
        	if ( look_xmit() )
			log_info("receiving a message from dh6scc");
#endif
                if ( msg.cmd==ADM_CMD_STOP ) {
			ifi_kill_dhcpv6();
			quit_flag = 1;
			break;
		}

                if (msg.cmd == ADM_OPEN_DHCPV6) {
			ifi = seek_ifi_byname(msg.un.data, msg.len);
                	if (ifi == NULL) {
                        	log_warn(1009,"unknown interface");
                         	continue;
			}
			open_dhcpv6(ifi);
			continue;
                }
                log_warn(1011,"unknown command %u", msg.cmd);

        }
	close(p->fd); p->fd=-1;
	del_fdp(p);
	free(p);
}

static void exit_dhcpv6_proc( pid_t pid )
{
	struct if_info *ifi = seek_ifi_bypid(pid);
	if (ifi != NULL)
		ifi->ifi_dh6 = 0;
}

void su_icmp_msgrcv( void * data )
{
	struct fd_list *p = (struct fd_list *)data;
        struct iovec aiov[1];
        struct msghdr msg;
        struct cmsghdr *cmsg;
        struct sockaddr_in6 sin6;
        struct icmpv6 *icmp_hdr;
        struct if_info *ifi;
        u_long len;
	u_int8_t ctrldata[512];
	u_int8_t packet[1024];
	struct in6_pktinfo *pktinfo;

        msg.msg_name = (caddr_t)&sin6;
        msg.msg_namelen = sizeof(struct sockaddr_in6) ;
        msg.msg_iov = aiov;
        msg.msg_iovlen = 1;
        aiov[0].iov_base = packet;
        aiov[0].iov_len = sizeof(packet);
        msg.msg_control = ctrldata;
        msg.msg_controllen = sizeof(ctrldata);
        msg.msg_flags = 0;
        if ((len = recvmsg(p->fd, &msg, 0)) < 0)
                log_sys(1012, "recvmsg");
        cmsg = (struct cmsghdr *)ctrldata;

	if ((pktinfo=cmsg_get(&msg, IPPROTO_IPV6, IPV6_RXINFO)) == NULL) {
		log_warn(1012, "IPV6_RXINFO not found");
		return;
	}
        if ( (ifi=seek_ifi_byindex(pktinfo->ipi6_ifindex)) == NULL ) {
		log_warn(1013, "interface not found");
		return;
	}
   
        icmp_hdr = (struct icmpv6 *)(packet + sizeof(struct ipv6));
        if (!ICMPV6_INFOTYPE(icmp_hdr->icmp6_type))
                return;
        if (icmp_hdr->icmp6_type = ICMPV6_ADVERTISMENT_RT) {
		pid_t pid;
#ifdef DH6_DEBUG
        	if ( look_tm() ) {
			log_info("Receiving Router Advertisement");
			log_info("Mbit: %u,  Obit: %u",
				rtadv_managed_bit(icmp_hdr),
				rtadv_other_bit(icmp_hdr));
			/* log_info("Quaizar, r u busy?");
			log_info("Don't work too much"); */
		}
#endif
                if ( (rtadv_managed_bit(icmp_hdr) ||
                      rtadv_other_bit(icmp_hdr)) &&
		     !ifi->ifi_dh6 ) 
			open_dhcpv6(ifi);
        }
}

int open_dhcpv6(struct if_info *ifi)
{
	pid_t 	pid;
	ifi->ifi_dh6 = 1;
	end_timing();
	signal(SIGCHLD, exit_dhcpv6_proc);
	if ( (pid=fork()) < 0 )
		log_sys(1013, "fork");
	else if ( pid==0 ) {
#ifdef 0 /* this descriptor is not queued to dispatch queue */
		del_fdp(&unix_sock); 
		clean_fd_list();
		begin_timing();
#endif
		close(unix_sock.fd);
		dh6_client_main( ifi->ifi_ifindex );
	}
	ifi->ifi_pid = pid;
	begin_timing();
	return 0;
}
