/*
 * su_clicmd.c
 *
 * errors 1200 - 1299
 */

#include "dh6all.h"
#include "localdefs.h"
#include "u_timer.h"
#include "dispatch.h"
#include "u_buf.h"
#include "dh6.h"
#include "ext6.h"
#include "nd6_if.h"
#include "dh6_cli.h"
#include "dh6_adm.h"
#include "sock.h"

extern char *optarg;
extern int optind, opterr, optopt;

int debug = 0;

/*
 *  Generate an error message with the printf facility.
 */
void p_error (int err, const char *ctl, ...)
{
    	char buf[1000];
    	va_list list;
    	va_start (list,ctl);
    	vsprintf (buf,ctl,list);
    	printf("error %d: %s\n", err, buf);
}


main( int argc, char * argv[] )
{
	u_int8_t buf[MAX_MSG_LEN];
	DH6_MESSAGE *msg=(DH6_MESSAGE *)buf;
	struct sockaddr_un  serv_addr;
	int  sockfd, servlen, msglen;

	if ( argc<2 )
		scmd_usage();

	if ( strcmp(argv[1],"start")==0 )
	{
		system("dh6cd");
		exit(0);
	}
	else if ( strcmp(argv[1],"restart")==0 )
	{
		char cmd[50];
		sprintf(cmd, "%s stop", argv[0]);
		system(cmd);
		sleep(1);
		sprintf(cmd, "%s start", argv[0]);
		system(cmd);
		exit(0);
	}

	else if ( strcmp(argv[1],"open")!=0 &&
	          strcmp(argv[1],"close")!=0 &&
	          strcmp(argv[1],"stop")!=0 )
		scmd_usage();

	/*
	 * 	Fill in the structure "serv_addr" with the address of the
	 * 	mip server that we want to send to.
	 */

	bzero((char *) &serv_addr, sizeof(serv_addr));
	serv_addr.sun_family = AF_UNIX;
	strcpy(serv_addr.sun_path, dh6cli_adm_name());
	servlen = strlen(serv_addr.sun_path) + sizeof(serv_addr.sun_family) + 1;

	/*
	 * 	Open a socket (an UNIX domain stream socket).
	 */

	if ( (sockfd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
		p_error(1301, strerror(errno));

	/*
	 * 	Connect to the server.
	 */

	if (connect(sockfd, (struct sockaddr *) &serv_addr, servlen) < 0)
		p_error(1302, strerror(errno));

	memset(buf, 0, MAX_MSG_LEN);
	if ( strcmp(argv[1],"open")==0 )
	{
   		msg->adm.type   = ADM_COMMAND;
   		msg->adm.cmd    = ADM_OPEN_DHCPV6;
   		msg->cmd_nm.len = strlen(argv[2]);
   		strcpy(msg->cmd_nm.name, argv[2]);

		msglen = 3 + msg->cmd_nm.len;
		scmd_sendto_serv( sockfd, msg, msglen );
		goto out;
	}
	if ( strcmp(argv[1],"close")==0 )
	{
   		msg->adm.type   = ADM_COMMAND;
   		msg->adm.cmd    = ADM_CLOSE_DHCPV6;
   		msg->cmd_nm.len = strlen(argv[2]);
   		strcpy(msg->cmd_nm.name, argv[2]);

		msglen = 3 + msg->cmd_nm.len;
		scmd_sendto_serv( sockfd, msg, msglen );
		goto out;
	}
    	if ( strcmp(argv[1],"stop")==0 )
    	{
   		msg->adm.type   = ADM_COMMAND;
   		msg->adm.cmd    = ADM_CMD_STOP;
		msglen = 2;
		scmd_sendto_serv( sockfd, msg, msglen );
		goto out;
    	}

out:
	close(sockfd);
	exit(0);
}

scmd_usage( void )
{
	printf("usage: nd6cd start\n");
	printf("       nd6cd restart\n");
	printf("       nd6cd stop\n");
	printf("       nd6cd open <interface>\n");
	printf("       nd6cd close <interface>\n");
	exit(1);
}

/*
 *	send a message to server
 */

int scmd_sendto_serv( int sockfd, DH6_MESSAGE *msg, int msglen )
{
	if ( send(sockfd, msg, msglen, 0) != msglen)
	{
		p_error(603, strerror(errno));
		return -1;
	}

	return 0;
}
