/*
 * svr_conf.c   DHCPv6 Server Configuration Routines
 *
 * Version:     @(#)svr_conf.c   1.0     Dec-8-96
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed WITHOUT ANY WARRANTY and even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Many thanks to Jim Bound in Digital Equipment Corp.
 * Appreciate any further work on this release.
 *
 * Authors:     Yunzhou Li (yunzhou@iol.unh.edu)
 *
 */

#include "compat.h"
#include <db.h>
#include "localdefs.h"
#include "u_timer.h"
#include "u_buf.h"
#include "dispatch.h"
#include "dh6.h"
#include "ext6.h"
#include "dh6_adm.h"
#include "dh6_db.h"
#include "parse.h"
#include "dh6_if.h"
#include "dh6_svr.h"

extern int	MaxPreferLife;
extern int	MaxValidLife;

extern struct in6_addr MulticastAddr;

int read_dh6_server_conf( void )
{
	char fname[60];
	char buf[100];
	int type;
	FILE *f;

	sprintf(fname, "%s%s", SVR_CONF_PATH, DH6SVR_CONF);
	if ((f=fopen(fname, "r+")) == NULL)
		log_sys(3301, "fopen");

	rewind(f);
	parse_init();
	do {
		type = get_next_token(f, buf);
		switch (type) {
		case SVR_MAX_PREFER_LIFE:
			type = get_next_token(f, buf);
			if (type & DIGIT) {
				MaxPreferLife = atoi(buf);
				continue;
			}
			log_warn(3302, "incorrect MaxPreferLife format");
			break;
		case SVR_MAX_VALID_LIFE:
			type = get_next_token(f, buf);
			if (type & DIGIT) {
				MaxValidLife = atoi(buf);
				continue;
			}
			log_warn(3303, "incorrect MaxValidLife format");
			break;
		case SVR_MULTICAST_ADDR:
			type = get_next_token(f, buf);
			if (type & STRING) {
				inet_pton(AF_INET6, buf, &MulticastAddr);
				continue;
			}
			log_warn(3304, "incorrect MULTICAST_ADDR format");
			break;
		}
	} while (type != EOF);

	fclose(f);
	return 0;
}

int seek_match( char *id, int type )
{
	if ( !isascii(id[0]) )
		return type;

#ifdef DH6_DEBUG
	if ( look_db_more() )
		log_info("svr_conf: id = %s", id);
#endif
	switch ( tolower(id[0]) ) {
		case 'm':
			if (!strcasecmp (id, "MaxPreferLife"))
				return SVR_MAX_PREFER_LIFE;
			if (!strcasecmp (id, "MaxValidLife"))
				return SVR_MAX_VALID_LIFE;
			if (!strcasecmp (id, "MulticastAddr"))
				return SVR_MULTICAST_ADDR;
		case 'f':
			if (!strcasecmp (id, "From"))
				return SVR_ADDR_FROM;
		case 't':
			if (!strcasecmp (id, "To"))
				return SVR_ADDR_TO;
		case 'p':
			if (!strcasecmp (id, "Prefix"))
				return SVR_ADDR_PREFIX;
	}
	return type;
}
