/*
 * u_buf.h    	DHCPv6 Buffer Structures
 *
 * Version:     @(#)u_buf.h   1.0     Dec-8-96
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed WITHOUT ANY WARRANTY and even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Many thanks to Jim Bound in Digital Equipment Corp.
 * Appreciate any further work on this release.
 *
 * Authors:     Yunzhou Li (yunzhou@iol.unh.edu)
 *
 */

#ifndef __U_BUF_H
#define __U_BUF_H

struct chain {
    struct chain	*next;
    struct chain	*prev;
#define MAX_SIZE_TABLE	256
#define MAX_TABLES	50
    u_int8_t		__[MAX_SIZE_TABLE-sizeof(void *)*2];
};

struct large_chain {
    struct large_chain	*next;
    struct large_chain	*prev;
#define MAX_SIZE_LARGE_TABLE	1024
#define MAX_LARGE_TABLES	5
    u_int8_t		____[MAX_SIZE_LARGE_TABLE-sizeof(void *)*2];
};

typedef union {
    struct {
    	struct chain		*head;
    	struct chain		*tail;
    } chain;
    struct {
    	struct large_chain	*head;
    	struct large_chain	*tail;
    } large_chain;
    struct {
    	struct if_info		*head;
    	struct if_info		*tail;
    } ifi;
    struct {
    	struct cli_relay	*head;
    	struct cli_relay	*tail;
    } cr;
    struct {
    	struct cli_server	*head;
    	struct cli_server	*tail;
    } cs;
    struct {
    	struct cli_ifaddr	*head;
    	struct cli_ifaddr	*tail;
    } cia;
    struct {
    	struct svr_client	*head;
    	struct svr_client	*tail;
    } sc;
    struct {
    	struct svr_ifaddr	*head;
    	struct svr_ifaddr	*tail;
    } sia;
    struct {
    	struct rly_server	*head;
    	struct rly_server	*tail;
    } rs;
    struct {
    	struct rly_ifaddr	*head;
    	struct rly_ifaddr	*tail;
    } ria;
} UNION_LIST;

#define ul_head		chain.head
#define ull_head	large_chain.head
#define ifi_head	ifi.head
#define cr_head		cr.head
#define cs_head		cs.head
#define cia_head	cia.head
#define sc_head		sc.head
#define sia_head	sia.head
#define rs_head		rs.head
#define ria_head	ria.head

extern void u_queue_head(UNION_LIST *, void *);
extern void u_queue_tail(UNION_LIST *, void *);
extern void u_fore_queue_mid(UNION_LIST *, void *, void *);
extern void u_hind_queue_mid(UNION_LIST *, void *, void *);
extern void u_unlink(UNION_LIST *, void *);

extern void * u_clone_node( void *, int );
extern void * u_alloc_node( void );
extern void u_free_node( void * );

extern void * u_clone_large_node( void *, int );
extern void * u_alloc_large_node( void );
extern void u_free_large_node( void * );

#endif /* __U_BUF_H */
