/*
 * u_timer.h    DHCPv6 Timer Structures
 *
 * Version:     @(#)u_timer.h   1.0     Dec-8-96
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed WITHOUT ANY WARRANTY and even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Many thanks to Jim Bound in Digital Equipment Corp.
 * Appreciate any further work on this release.
 *
 * Authors:     Yunzhou Li (yunzhou@iol.unh.edu)
 *
 */

#ifndef __U_TIMER_H
#define __U_TIMER_H

#ifdef MORE_FREQUENT

#define TIMER_INTERVAL	100	/* in milli-seconds */
#define SECONDS		10	/* a number of interval in one second */
#define MILLI_SECS	1	/* a number of interval in one milli-second */

#else

#define TIMER_INTERVAL	1000	/* in milli-seconds */
#define SECONDS		1	/* a number of interval in one second */
#define MILLI_SECS	1	/* a number of interval in one milli-second */

#endif

struct u_timer {
        struct u_timer *next;
        struct u_timer *prev;
        unsigned long expires;
        void	*data;
        void (*function)(void *);
};

struct poll_list {
        struct poll_list *next;
        struct poll_list *prev;
        void	*data;
        void (*function)(void *);
};

extern void begin_timing( void );
extern void end_timing( void );
extern void init_timer(struct u_timer *);
extern int stop_timer(struct u_timer *);
extern void start_timer(struct u_timer *, u_int16_t);
extern int restart_timer(struct u_timer *, u_int16_t);
extern void run_timer_table( int );

extern void init_poll(struct poll_list *);
extern void add_poll(struct poll_list *);
extern void del_poll(struct poll_list *);

extern void lock_timer_table(void);
extern void unlock_timer_table(void);
extern int  timer_table_locked(void);
extern void u_ignore( void * );

#endif /* __U_TIMER_H */
